@@ -434,7 +434,7 @@ NSLog(@"Callback");
   DWORD wstyle = 0;
         
   if ([self handlesWindowDecorations] == NO)
-    return WS_POPUP;
+    return WS_POPUP | WS_CLIPCHILDREN;
         
   if (style == 0)
     wstyle = WS_POPUP;
@@ -1563,6 +1563,11 @@ NSLog(@"Callback");
   SetCursor((HCURSOR)cid);
 }
 
+- (void) setParentWindow: (int)parentWin 
+          forChildWindow: (int)childWin
+{
+  //SetParent((HWND)childWin, (HWND)parentWin);
+}
 
 @end
 
@@ -1663,7 +1668,8 @@ process_key_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
 
   GetKeyboardState(keyState);
   eventFlags = 0;
-  if (keyState[VK_CONTROL] & 128)
+  /* AltGr is mapped to right alt + left control */ 
+  if ((keyState[VK_CONTROL] & 128) && !((keyState[VK_LCONTROL] & 128) && (keyState[VK_RMENU] & 128))) 
     eventFlags |= NSControlKeyMask;
   if (keyState[VK_SHIFT] & 128)
     eventFlags |= NSShiftKeyMask;
