@@ -93,7 +93,7 @@ LRESULT CALLBACK win32SubwindowProc(
   NSDebugMLLog(@"WGL", @"MS window class initialized (%u)", atom);
 }
 
-- (id) initWithView: (NSOpenGLView *) view
+- (id) initWithView: (NSOpenGLView *)view
 {
   NSRect rect;
   WIN32Server *server;
@@ -102,35 +102,43 @@ LRESULT CALLBACK win32SubwindowProc(
   HINSTANCE hInstance;
   WNDCLASS wclss;
   ATOM atom;
+  RECT parent_rect;
 
   self = [super init];
-  if(self)
-  {
-    attached = (NSOpenGLView*)view;
+  if (!self)
+    return nil;
 
-    win = [view window];
-    NSAssert(win, @"request of a window attachment on a view that is not on a NSWindow");
+  attached = (NSOpenGLView*)view;
 
-    if ([view isRotatedOrScaledFromBase])
-      [NSException raise: NSInvalidArgumentException
-	 	  format: @"Cannot attach an window to a view that is rotated or scaled"];
+  win = [view window];
+  NSAssert(win, @"request of a window attachment on a view that is not on a NSWindow");
   
-    server = (WIN32Server *)GSServerForWindow(win);
-    NSAssert(server != nil, NSInternalInconsistencyException);
+  if ([view isRotatedOrScaledFromBase])
+    [NSException raise: NSInvalidArgumentException
+                 format: @"Cannot attach an window to a view that is rotated or scaled"];
+  
+  server = (WIN32Server *)GSServerForWindow(win);
+  NSAssert(server != nil, NSInternalInconsistencyException);
 
-    NSAssert([server isKindOfClass: [WIN32Server class]], 
-	NSInternalInconsistencyException);
+  NSAssert([server isKindOfClass: [WIN32Server class]], 
+           NSInternalInconsistencyException);
 
-    if ([server handlesWindowDecorations] == YES)
+  hInstance = GetModuleHandle(NULL);
+
+  /* Grab the window class we have registered on [+initialize] */
+  atom = GetClassInfo( hInstance, NSOPENGLSUBWINDOWCLASS, &wclss );
+  NSAssert(atom, @"MS window class not found !");
+
+  GetClientRect((HWND)[win windowNumber], &parent_rect);
+  if ([server handlesWindowDecorations] == YES)
     {
       /* The window manager handles window decorations, so the
        * the parent X window is equal to the content view and
        * we must therefore use content view coordinates.
        */
-      rect = [view convertRect: [view bounds]
-		        toView: [[view window] contentView]];
+      rect = [view convertRect: [view bounds] toView: [[attached window] contentView]];
     }
-    else
+  else
     {
       /* The GUI library handles window decorations, so the
        * the parent X window is equal to the NSWindow frame
@@ -138,33 +146,23 @@ LRESULT CALLBACK win32SubwindowProc(
        */
       rect = [view convertRect: [view bounds] toView: nil];
     }
-	  
-    hInstance = GetModuleHandle(NULL);
-
-    /* Grab the window class we have registered on [+initialize] */
-    atom = GetClassInfo( hInstance, NSOPENGLSUBWINDOWCLASS, &wclss );
-    NSAssert(atom, @"MS window class not found !");
-
-    RECT parent_rect;
-    GetClientRect((HWND)[win windowNumber], &parent_rect);
-
-    x = NSMinX(rect);
-    y = (parent_rect.bottom - parent_rect.top) - NSMaxY(rect);
-    width = NSWidth(rect);
-    height = NSHeight(rect);
+  x = NSMinX(rect);
+  y = (parent_rect.bottom - parent_rect.top) - NSMaxY(rect);
+  width = NSWidth(rect);
+  height = NSHeight(rect);
   
-    NSDebugMLLog(@"WGL", @"MS window creation (%d, %d, %u, %u)", x, y, width, height);
+  NSDebugMLLog(@"WGL", @"MS window creation (%d, %d, %u, %u)", x, y, width, height);
+  
+  winid = CreateWindow(
+      NSOPENGLSUBWINDOWCLASS, NSOPENGLSUBWINDOWNAME, 
+      WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE, 
+      x, y, width, height, 
+      (HWND)[win windowNumber], (HMENU)NULL, hInstance, (LPVOID)self);
 
-    winid = CreateWindow(
-	NSOPENGLSUBWINDOWCLASS, NSOPENGLSUBWINDOWNAME, 
-	WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE, 
-	x, y, width, height, 
-	(HWND)[win windowNumber], (HMENU)NULL, hInstance, (LPVOID)self);
+  NSAssert(winid, @"Failed to create a MS window");
+  
+  ShowCursor( TRUE );
 
-    NSAssert(winid, @"Failed to create a MS window");
-
-    ShowCursor( TRUE );
-  }
   return self;
 }
 
@@ -408,6 +406,7 @@ static Win32GLContext *currentGLContext;
 - (void)setView:(NSView *)view
 {
   Win32Subwindow *win;
+  NSView *current_view;
   
   if (!view)
     [NSException raise: NSInvalidArgumentException
@@ -415,7 +414,14 @@ static Win32GLContext *currentGLContext;
 
   NSAssert(format, NSInternalInconsistencyException);
   win = [Win32Subwindow subwindowOnView: (NSOpenGLView*) view];
+  current_view = [self view];
+  if ( current_view != nil )
+    {
+      [current_view _setIgnoresBacking: saved_ignores_backing];
+    }
   ASSIGN(wsubwin, win);
+  saved_ignores_backing = [view _ignoresBacking];
+  [view _setIgnoresBacking: YES];
 
 //   {
 //     GLXFBConfig  *conf_tab;
@@ -541,17 +547,20 @@ LRESULT CALLBACK win32SubwindowProc(
         return TRUE;
       }
       break;*/
-/*    case WM_PAINT:
+    case WM_PAINT:
       {
-        PAINTSTRUCT ps;
-        BeginPaint(hWnd, &ps);
-        if ((wsubwin->context->wgl_context) {
-          redraw();
-        }
-        EndPaint(hWnd, &ps);
-        return 0;
+        RECT wr;
+        RECT ir;
+        
+        if ( GetWindowRect(hWnd,&wr) && GetUpdateRect(hWnd, &ir, FALSE) )
+          {
+            ir.top  += wr.top;
+            ir.left += wr.left;
+            InvalidateRect( GetParent(hWnd), &ir,FALSE );
+          }
+        return DefWindowProc(hWnd, message, wParam, lParam);
       }
-      break;*/
+      break;
 /*    case WM_CHAR:
       // handle keyboard input
       switch ((int)wParam) {
